@@ -19,13 +19,16 @@
 #'
 #' This function is one of the methods of
 #' \code{add_quantile}. Currently, you can only use this function to
-#' compute the quantiles of the response of a Poisson regression with
-#' the \eqn{\log}-link function.
-#'
-#' Quantiles of generalized linear models are determined by
-#' \code{add_quantile} through a simulation using \code{arm::sim}.
-#'
+#' compute the quantiles of the response of Poisson, Quasipoisson, or
+#' Gamma regression models.  Quantile estimates for Bernoulli response
+#' variables (i.e., logistic regression) are not supported.
 #' 
+#' Quantiles of generalized linear models are determined by
+#' \code{add_quantile} through a simulation using \code{arm::sim}. If
+#' a Quasipoisson regression model is fit, simulation using the
+#' Negative Binomial distribution is performed, see Gelman and Hill
+#' (2007).
+#'
 #' @param tb A tibble or data frame of new data.
 #' @param fit An object of class \code{glm}. Predictions are made with
 #'     this object.
@@ -59,11 +62,12 @@
 #' # the number of simulations to run, and give the vector of
 #' # quantiles a custom name.
 #' add_quantile(cars, fit, p = 0.5, name = "my_quantile", nSims = 300)
+#'
 #' 
 #' @export
 
 add_quantile.glm <- function(tb, fit, p, name = NULL, yhatName = "pred",
-                             nSims = 200, ...){
+                             nSims = 2000, ...){
     if (p <= 0 || p >= 1)
         stop ("p should be in (0,1)")
     if (is.null(name))
@@ -71,36 +75,32 @@ add_quantile.glm <- function(tb, fit, p, name = NULL, yhatName = "pred",
     if ((name %in% colnames(tb))) {
         warning ("These quantiles may have already been appended to your dataframe. Overwriting.")
     }
+
     if (fit$family$family == "binomial"){
-       stop ("Quantiles for Logistic Regression don't make sense") 
-    }
-    if (fit$family$family == "poisson"){
-        warning ("The response is not continuous, so estimated quantiles are only approximate")
-        sim_quantile_pois(tb, fit, p, name, yhatName, nSims)
+        if(max(fit$prior.weights) == 1)
+        stop("Prediction intervals for Bernoulli response variables aren't useful") else {
+          warning("Treating weights as indicating the number of trials for a binomial regression where the response is the proportion of successes")
+          warning("The response variable is not continuous so Prediction Intervals are approximate")
+        }
     }
+
+    if (fit$family$family %in% c("poisson", "qausipoisson"))
+        warning("The response is not continuous, so estimated quantiles are only approximate")
+
+    if(!(fit$family$family %in% c("poisson", "quasipoisson", "Gamma", "binomial")))
+        stop("Unsupported family")
+
+    sim_quantile_other(tb, fit, p, name, yhatName, nSims)
 }
 
-sim_quantile_pois <- function(tb, fit, p, name, yhatName, nSims){
-    nPreds <- NROW(tb)
-    modmat <- model.matrix(fit)
-    response_distr <- fit$family$family
-    inverselink <- fit$family$linkinv
+sim_quantile_other <- function(tb, fit, p, name, yhatName, nSims){
+
     out <- predict(fit, newdata = tb, type = "response")
-    sims <- arm::sim(fit, n.sims = nSims)
-    sim_response <- matrix(0, ncol = nSims, nrow = nPreds)
-
-    for (i in 1:nPreds){
-        if(response_distr == "poisson"){
-            sim_response[i,] <- rpois(n = nSims, lambda = inverselink(rnorm(nPreds,sims@coef[i,] %*% modmat[i,], sd = sims@sigma[i])))
-            }
-    }
-
+    sim_response <- get_sim_response(tb, fit, nSims)
     quants <- apply(sim_response, 1, FUN = quantile, probs = p, type = 1)
 
     if(is.null(tb[[yhatName]]))
         tb[[yhatName]] <- out
     tb[[name]] <- quants
     tibble::as_data_frame(tb)
-
-
 }
